#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later
"""Command line interfaces for MDAnalysis analysis classes."""

from mdacli.cli import cli

__all__ = ["cli"]

__version__ = '0.1.12'



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later
"""Logging."""

import contextlib
import logging
import sys

from .colors import Emphasise


@contextlib.contextmanager
def setup_logging(logobj, logfile=None, debug=False):
    """
    Create a logging environment for a given logobj.

    Parameters
    ----------
    logobj : ``logging.Logger``
        A logging instance
    logfile : str
        Name of the log file
    debug : bool
        If ``True`` detailed debug logs inludcing filename and function name
        are displayed. If ``False`` only the message logged from
        errors, warnings and infos will be displayed.
    """
    try:
        format = '{message}'
        if debug:
            format = "[{levelname}] {filename}:{name}:{funcName}:{lineno}: " \
                     + format
            level = logging.DEBUG
        else:
            level = logging.INFO

        logging.basicConfig(format=format,
                            handlers=[logging.StreamHandler(sys.stdout)],
                            level=level,
                            style='{')

        if logfile:
            logfile += ".log" * (not logfile.endswith("log"))
            handler = logging.FileHandler(filename=logfile, encoding='utf-8')
            handler.setFormatter(logging.Formatter(format, style='{'))
            logobj.addHandler(handler)
        else:
            logging.addLevelName(logging.INFO, Emphasise.info("INFO"))
            logging.addLevelName(logging.DEBUG, Emphasise.debug("DEBUG"))
            logging.addLevelName(logging.WARNING, Emphasise.warning("WARNING"))
            logging.addLevelName(logging.ERROR, Emphasise.error("ERROR"))
            logobj.info('Logging to file is disabled.')

        yield
    finally:
        handlers = logobj.handlers[:]
        for handler in handlers:
            handler.close()
            logobj.removeHandler(handler)



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later

"""Useful helper functions for running the cli."""

import inspect
import re
import sys
from collections import defaultdict

import MDAnalysis as mda


def _exit_if_a_is_b(obj1, obj2, msg):
    """Exit if `obj1` and `obj2` are the same."""
    if obj1 is obj2:
        sys.exit(msg)


def split_time_unit(s):
    """
    Split time and units.

    Follows the regex:: https://regex101.com/r/LZAbil/2

    Returns
    -------
    tuple (float, str)
        Value as float, units as str.

    Raises
    ------
    IndexError
        Tuple could not be found. This happens when a number is not
        present in the start of the string.
    """
    type_regex = re.compile(r'^(\-?\d+\.?\d*|\-?\.\d+|\-?\.?\d+[eE]\-?\d+|-?\d+\.?\d*[eE]\d+)($|[a-z]*$)')  # noqa: E501
    value, unit = type_regex.findall(s)[0]
    return float(value), unit


def convert_str_time(x, dt):
    """
    Convert a string `x` into a frame number based on given `dt`.

    If `x` does not contain any units its assumed to be a frame number
    already.

    See :func:`split_time_unit`.

    Parameters
    ----------
    x : str
        the input string
    dt : float
        the time step in ps

    Returns
    -------
    int
        frame number

    Raises
    ------
    ValueError
        The input does not contain any units but is not an integer.
    """
    val, unit = split_time_unit(x)
    if unit != "":
        val = mda.units.convert(val, unit, "ps")
        return int(val // dt)
    elif val % 1 != 0:  # the number is not int'able
        raise ValueError(
            "Only integers or time step combinations (´12ps´) "
            "are valid for frame selection"
            )
    else:
        return int(val)


def parse_callable_signature(callable_obj):
    """
    Parse a callable signature to a convenient dictionary for CLI creation.

    The parameters used in the CLI are a combination of the callable
    signature and the information in the callable docstring.

    Parameters
    ----------
    callable_obj : callable
        The callable object to inspect. Details of this object required
        for the creation of a CLI are added to the `storage_dict`.

    Returns
    -------
    dict
    """
    storage_dict = {}
    storage_dict["callable"] = callable_obj

    sig = inspect.signature(callable_obj)
    summary, summary_extended, doc = parse_docs(callable_obj)

    # args for CLIs
    positional_args = {}
    optional_args = {}

    # tuple to ignore args and kwargs, these are note necessary to consider
    # for the purpose of this implementation - at least to currently
    ARGS_KWARGS = (
        inspect.Parameter.VAR_KEYWORD,
        inspect.Parameter.VAR_POSITIONAL,
        )

    # brings to local scope
    EMPTY = inspect.Parameter.empty

    # for each parameter in the callable's signature
    for sig_name, sig_param in sig.parameters.items():

        if sig_param.kind in ARGS_KWARGS:
            pass

        # positional parameter
        elif sig_param.default == EMPTY:

            # parameter type and description is extract from docstring
            for param_name, doc_param in doc.items():
                if param_name == sig_name:
                    positional_args[sig_name] = {
                        "type": doc_param['type'],
                        "desc": doc_param['desc'],
                        }
                    break  # done, jumps off the loop

            else:
                # else reaches if the parameter in the signature is not present
                # in the docstring. It shouldn't, but just in case :-)
                # unless we explicitly decide not to consider any parameters
                # not referenced in the documentation, this should be kept
                #
                # str is the default value of argparse arguments type parameter
                positional_args[sig_name] = {
                    "type": "str",
                    "desc": "No description available.",
                    }

        # named parameters
        else:
            for param_name, doc_param in doc.items():
                if param_name == sig_name:
                    optional_args[sig_name] = {
                        # type taken form docstring
                        "type": doc_param['type'],
                        # but default taken from signature
                        "default": sig_param.default,
                        "desc": doc_param['desc'],
                        }
                    break  # parameter captured, break the loop
            else:
                # if the parameter is in signature but NOT in the docstring
                # uses type.__name__ to match with STR_TYPE_DICT keys
                optional_args[sig_name] = {
                    "type": type(sig_param.default).__name__,  # corrected here
                    "default": sig_param.default,
                    "desc": "No description available.",
                    }

    # places all information captured for the callable in the dictionary
    storage_dict["positional"] = positional_args
    storage_dict["optional"] = optional_args
    storage_dict["desc"] = summary
    storage_dict["desc_long"] = summary_extended

    return storage_dict


def parse_docs(klass):
    """
    Parse classes docstrings to a convenient dictionary.

    This parser is based on NumpyDocString format, yet it is not so
    strict. Combined docstrings from class main docstring and `__init__`
    method.

    Parameters
    ----------
    klass : callable
        A klass object from which a DOCSTRING can be extracted.

    Returns
    -------
    tuple (str, str, dict of dict)
        * One line summary description of the callable
        * Extended description of the callable
        * dictionary where keys are parameter names and subdictinary
            has keys "type" and "desc" for parameter type and description.
    """
    doc = klass.__doc__ or ''
    doc += klass.__init__.__doc__ or ''

    doc_lines = [s for s in (s.strip() for s in doc.lstrip().split('\n')) if s]

    # first docstring sentence is the summary
    summary = doc_lines[0]

    # sometimes signature parameters in docstring are referred as "Arguments"
    try:
        param_index = doc_lines.index('Parameters')
    except ValueError:
        param_index = doc_lines.index('Arguments')

    # the extended summary is every text that exists between the summary
    # and the Parameters title line
    summary_extended = '\n'.join(doc_lines[1:param_index])

    # the line to start collecting parameters is `param_index` + 2
    # because of the "Parameters" title and the '---------' underscore
    par_i = param_index + 2

    # search for the line where Parameters section ends
    # will search until the end of the docstring or until a '----'-like
    # line is found - corresponding to the start of another section
    for i, line in enumerate(doc_lines[par_i:], start=par_i):
        if '----' in line:  # at least of Note\n----
            # -1 because the exact line is the one of the title not of
            # the underlines
            end_param_line = i - 1
            break
    else:
        # if the end of the docstring is reached, takes the last line
        end_param_line = -1

    # starts collecting parameters in doctrings
    params = defaultdict(dict)  # the dictionary which will be returned

    # temporary parameter description list
    desc_tmp = []

    # regex to find parameter types
    type_regex = re.compile(r'^([\[\]\w]+|\{.*\})|(?<=\`\~)(.*?)(?=\`)')

    # goes back to front to register descriptions first ;-)
    # considers only the Parameters section
    for line in doc_lines[par_i: end_param_line][::-1]:
        if ' : ' in line:
            par_name, others_ = line.split(' : ')
            par_type = [_ for _ in type_regex.findall(others_)[0] if _][0]
            params[par_name]['type'] = par_type
            params[par_name]['desc'] = ' '.join(desc_tmp[::-1])
            desc_tmp.clear()
        else:
            desc_tmp.append(line)

    return summary, summary_extended, params



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later

"""
A command line interface (CLI) to the analysis modules of MDAnalysis.

The modules are all structured as part of a single mdacli wrapper, and invoked
with commands like `mda RMSD`. This command uses the class
:class:`MDAnalysis.analysis.rms.RMSD` for calculating the RMSD.
Documentation for each module can be found at the respective sections on the
`MDAnalysis Documentation`_, as well as
`mda command -h`.

.. _`MDAnalysis Documentation`:
   https://docs.mdanalysis.org/stable/documentation_pages/analysis_modules.html
"""
from MDAnalysis.analysis import __all__

import mdacli


def main():
    """Execute main CLI entry point."""
    # Thse module are currently not supported. Either due a different
    # structure or due parameters that aree not supported by our parser.
    skip_mods = ['AnalysisFromFunction',
                 'HydrogenBondAnalysis',
                 'WaterBridgeAnalysis',
                 'Contacts',
                 'Dihedral',
                 'PersistenceLength',
                 'InterRDF_s']

    mdacli.cli(name="MDAnalysis",
               module_list=[f'MDAnalysis.analysis.{m}' for m in __all__],
               version=mdacli.__version__,
               description=__doc__,
               skip_modules=skip_mods,
               ignore_warnings=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later

"""Functionalities that support the creation of the command lines interface."""
import argparse
import importlib
import inspect
import json
import logging
import os
import warnings
from typing import List

import MDAnalysis as mda
from MDAnalysis.transformations.boxdimensions import set_dimensions

from .colors import Emphasise
from .save import save
from .utils import convert_str_time, parse_callable_signature, parse_docs


logger = logging.getLogger(__name__)

# serves CLI factory
STR_TYPE_DICT = {
    "bool": bool,
    "str": str,
    "list": list,
    "tuple": tuple,
    "dict": dict,
    "int": int,
    "float": float,
    "complex": complex,
    "NoneType": None,
    "AtomGroup": mda.AtomGroup,
    "list[AtomGroup]": List[mda.AtomGroup],
    "Universe": mda.Universe,
    }


def _warning(message, *args, **kwargs):
    logger.warning(Emphasise.warning(message))


warnings.showwarning = _warning


class KwargsDict(argparse.Action):
    """
    Convert input string to a dictionary.

    If string points to a ".json" file, reads the file.
    Else, attempts to convert string to dictionary using json.loads.
    """

    def __call__(self, parser, namespace, value, option_string=None):
        """Call me."""
        if value.startswith("{") and value.endswith("}"):
            try:
                jdict = json.loads(value)

            except json.decoder.JSONDecodeError as err:
                raise json.decoder.JSONDecodeError(
                    "An error ocurred when reading "
                    f"{self.dest!r} argument.",
                    err.doc,
                    err.pos,
                    ) from None
        else:
            with open(value, 'r') as fin:
                jdict = json.load(fin)

        setattr(namespace, self.dest, jdict)


def find_classes_in_modules(cls, *module_names):
    """
    Find classes that belong to cls in modules.

    A series of names can be given as arguments.

    Parameters
    ----------
    cls : single class or list of classes
        parent reference class type to search for

    module_names : str
        module to import import in absolute or relative terms
        (e.g. either pkg.mod or ..mod).

    Returns
    -------
    list
    list of found class objects. If no classes are found, return None.
    """
    # Convert all cls to tuples
    if type(cls) not in (list, tuple):
        cls = [cls]
    members = []
    for name in module_names:
        module = importlib.import_module(name)
        for _, member in inspect.getmembers(module):
            if inspect.isclass(member) and issubclass(member, tuple(cls)) \
               and member not in cls:
                members.append(member)

    return members or None


def find_cls_members(cls,
                     modules,
                     ignore_warnings=False):
    """Find members of a certain class in modules.

    Parameters
    ----------
    cls : class or list of classes
        parent reference class or list of classes to be searched for
    modules : list
        list of modules for which members should be searched for
    ignore_warnings : bool
        Flag to ignore warnings
    """
    with warnings.catch_warnings():
        if not ignore_warnings:
            warnings.simplefilter('ignore')
        members = find_classes_in_modules(cls, *[m for m in modules])
    return members


def split_argparse_into_groups(parser, namespace):
    """
    Split the populated namespace of argparse into groups.

    https://stackoverflow.com/questions/31519997/is-it-possible-to-
    only-parse-one-argument-groups-parameters-with-argparse

    Parameters
    ----------
    parse : `argparse.ArgumentParser`
        argument parser instance
    namespace : `argparse.Namespace`
        instance storing the parameters

    Returns
    -------
    arg_grouped_dict : dict
        Dictionary containing parameters split according to their groups
    """
    arg_grouped_dict = {}
    for group in parser._action_groups:
        group_dict = {a.dest: getattr(namespace, a.dest, None)
                      for a in group._group_actions}
        arg_grouped_dict[group.title] = group_dict

    return arg_grouped_dict


def add_run_group(analysis_class_parser):
    """Add run group parameters to an given argparse.ArgumentParser instance.

    The run group adds the parameters `start`, `stop`, `step`, `verbose` to the
    parser.

    Parameters
    ----------
    analysis_class_parser : argparse.ArgumentParser
        The ArgumentsParser instance to which the run grorup is added
    """
    run_group = analysis_class_parser.add_argument_group(
        title="Analysis Run Parameters",
        description="Genereal parameters specific for running the analysis. "
                    "Parameters can be given in terms of frames (e.g. 12) "
                    "or as a time ('12ps'). Allowed time units are "
                    "'ps', 'fs' and 'ns'.")

    run_group.add_argument(
        "-b",
        dest="start",
        type=str,
        default="0",
        help="start frame or time for evaluation (default: %(default)s)"
        )

    run_group.add_argument(
        "-e",
        dest="stop",
        type=str,
        default="-1",
        help="end frame or time for evaluation (default: %(default)s)"
        )

    run_group.add_argument(
        "-dt",
        dest="step",
        type=str,
        default="1",
        help="step or time step for evaluation (default: %(default)s)"
        )

    run_group.add_argument(
        "-v",
        dest="verbose",
        help="Be loud and noisy",
        action="store_true",
        )


def add_output_group(analysis_class_parser):
    """Add output group parameters to argparse.ArgumentParser instance.

    The run group adds the parameters `output_prefix` and
    `output_directory` to the parser.

    Parameters
    ----------
    analysis_class_parser : argparse.ArgumentParser
        The ArgumentsParser instance to which the run grorup is added
    """
    output_group = analysis_class_parser.add_argument_group(
        title="Output Parameters",
        description="Genereal parameters specific for the result output.")

    output_group.add_argument(
        "-pre",
        dest="output_prefix",
        type=str,
        default="",
        help="Additional prefix for all output files. Files will be "
             " automatically named by the used module (default: %(default)s)"
        )

    output_group.add_argument(
        "-o",
        dest="output_directory",
        type=str,
        default=".",
        help="Directory in which the output files produced will be stored."
             "(default: %(default)s)"
        )


def add_cli_universe(parser, name=''):
    """Add universe parameters to an given argparse.ArgumentParser.

    instance. The parameters `topology`, `topology_format`, `atom_style`,
    `coordinates` and `trajectory_format` are added to the parse.

    Parameters
    ----------
    analysis_class_parser : argparse.ArgumentParser
        The ArgumentsParser instance to which the run grorup is added
    name : str
        suffix for the argument names
    """
    name = f'_{name}' if name else ''

    parser.add_argument(
        f"-s{name}",
        dest=f"topology{name}",
        type=str,
        default="topol.tpr",
        help="The topolgy file. "
        "The FORMATs {} are implemented in MDAnalysis."
        "".format(", ".join(mda._PARSERS.keys())),
        )

    parser.add_argument(
        f"-top{name}",
        dest=f"topology_format{name}",
        type=str,
        default=None,
        help="Override automatic topology type detection. "
        "See topology for implemented formats.")

    parser.add_argument(
        f"-atom_style{name}",
        dest=f"atom_style{name}",
        type=str,
        default=None,
        help="Manually set the atom_style information"
        "(currently only LAMMPS parser). E.g. atom_style='id type x y z'.")

    parser.add_argument(
        f"-f{name}",
        dest=f"coordinates{name}",
        type=str,
        default=None,
        nargs="+",
        help="A single or multiple coordinate files. "
        "The FORMATs {} are implemented in MDAnalysis."
        "".format(", ".join(mda._READERS.keys())),
        )

    parser.add_argument(
        f"-traj{name}",
        dest=f"trajectory_format{name}",
        type=str,
        default=None,
        help="Override automatic trajectory type detection. "
        "See trajectory for implemented formats.")

    parser.add_argument(
        f"-dimensions{name}",
        dest=f"dimensions{name}",
        type=float,
        default=None,
        nargs="+",
        help="Manually set/overwrite the simulation box dimensions to a "
        "vector containing unit cell dimensions "
        "[a, b, c, alpha, beta, gamma], lengths a, b, c are in angstrom, "
        "and angles alpha, beta, gamma are in degrees. "
        "Providing only three parameters will assume a rectengular simulation "
        "box (alpha = beta = gamma = 90°).")


def create_cli(sub_parser, interface_name, parameters):
    """
    Add subparsers to `cli_parser`.

    Subparsers parameters are divided in the following categories:

    2. Analysis Run parameters
         time frame as begin, end, step and vebosity

    3. Saving Parameters
        output_prefix and output_directory

    4. Mandatory Parameters
        mandatory parameters are defined in the CLI as named parameters
        as per design

    5. Optional Parameters
        Named parameters in the Analysis class

    6. Reference Universe Parameters
        A reference Universe for selection commands. Only is created if
        AtomGroup arguments exist.

    All CLI's parameters are named parameters.

    Parameters
    ----------
    sub_parser : argparse.sub_parser
        A sub parser where the new parser will be added.

    interface_name : str
        Name of the interface.

    parameters : dict
        Parameters needed to fill the argparse requirements for the
        CLI interface.

    Returns
    -------
    None
    """
    # creates the subparser
    analysis_class_parser = sub_parser.add_parser(
        interface_name,
        help=parameters["desc"],
        description=f"{parameters['desc']}\n\n{parameters['desc_long']}",
        formatter_class=argparse.RawDescriptionHelpFormatter,
        )

    # Add run_analsis function as the default func parameter.
    # this is possible because the run_analsis function is equal to all
    # Analysis Classes
    analysis_class_parser.set_defaults(
        analysis_callable=parameters["callable"])
    add_run_group(analysis_class_parser)

    # adds only if `save` method does not exist
    if not getattr(parameters['callable'], 'save', False):
        logger.debug("No save method found. Use generic one.")
        # TODO: add our save function as method. Avoids try except later...
        add_output_group(analysis_class_parser)
    else:
        # TODO: add parameters from save function to parser
        pass

    # add positional and optional arguments
    pos_ = sorted(list(parameters["positional"].items()), key=lambda x: x[0])
    opt_ = sorted(list(parameters["optional"].items()), key=lambda x: x[0])

    parameters_to_parse = pos_ + opt_
    mandatory_parameters_group = analysis_class_parser.add_argument_group(
        title="Mandatory Parameters",
        description="Mandatory parameters of this Analysis",
        )
    groups = len(pos_) * [mandatory_parameters_group]

    # Only create parser if optional arguments exist
    if len(opt_) > 0:
        optional_parameters_group = analysis_class_parser.add_argument_group(
            title="Optional Parameters",
            description="Optional parameters specific of this Analysis",
            )
        groups += len(opt_) * [optional_parameters_group]

    for group, (name, args_dict) in zip(groups, parameters_to_parse):

        # prepares parameters before add_argument
        try:
            type_ = STR_TYPE_DICT[args_dict["type"]]
        except KeyError:
            type_ = str

        try:
            default = args_dict["default"]
        except KeyError:
            default = None

        description = args_dict["desc"]
        flag = f"-{name}"
        arg_params = dict(dest=name,
                          help=description,
                          default=default,
                          )
        if type_ is dict:
            arg_params["default"] = None
            arg_params["action"] = KwargsDict
        elif type_ is bool:
            if default:
                flag = f"-no-{name}"
                arg_params["action"] = "store_false"
            else:
                arg_params["action"] = "store_true"
        elif type_ in (mda.AtomGroup, List[mda.AtomGroup]):
            if type_ == List[mda.AtomGroup]:
                arg_params["nargs"] = "+"

            arg_params["type"] = str
            arg_params["help"] += " Use a MDAnalysis selection string."

            # Create one reference Universe argument for atom selection
            try:
                reference_universe_group
            except NameError:
                reference_universe_group = \
                    analysis_class_parser.add_argument_group(
                        title="Reference Universe Parameters",
                        description="Parameters specific for loading "
                                    "the reference topology and trajectory"
                                    " used for atom selection.")
                add_cli_universe(reference_universe_group)

        elif type_ is mda.Universe:
            add_cli_universe(group, name)
            continue
        else:
            if type_ in (list, tuple):
                arg_params["nargs"] = "+"
            arg_params["type"] = type_
            arg_params["help"] += " (default: %(default)s)"

        group.add_argument(flag, **arg_params)
    return


def create_universe(topology,
                    coordinates=None,
                    topology_format=None,
                    trajectory_format=None,
                    atom_style=None,
                    dimensions=None):
    """
    Initilize a MDAnalysis universe instance.

    Parameters
    ----------
    topology : str, stream, `~MDAnalysis.core.topology.Topology`, `np.ndarray`
        A CHARMM/XPLOR PSF topology file, PDB file or Gromacs GRO file; used to
        define the list of atoms. If the file includes bond information,
        partial charges, atom masses, ... then these data will be available to
        MDAnalysis. Alternatively, an existing
        :class:`MDAnalysis.core.topology.Topology` instance may be given,
        numpy coordinates, or None for an empty universe.
    coordinates : str, stream, list of str, list of stream
        Coordinates can be provided as files of
        a single frame (eg a PDB, CRD, or GRO file); a list of single
        frames; or a trajectory file (in CHARMM/NAMD/LAMMPS DCD, Gromacs
        XTC/TRR, or generic XYZ format). The coordinates must be
        ordered in the same way as the list of atoms in the topology.
        See :ref:`Supported coordinate formats` for what can be read
        as coordinates. Alternatively, streams can be given.
    topology_format : str, None
        Provide the file format of the topology file; ``None`` guesses it from
        the file extension. Can also pass a subclass of
        :class:`MDAnalysis.topology.base.TopologyReaderBase` to define a custom
        reader to be used on the topology file.
    trajectory_format : str or list or object
        provide the file format of the coordinate or trajectory file;
        ``None`` guesses it from the file extension. Note that this
        keyword has no effect if a list of file names is supplied because
        the "chained" reader has to guess the file format for each
        individual list member [``None``]. Can also pass a subclass of
        :class:`MDAnalysis.coordinates.base.ProtoReader` to define a custom
        reader to be used on the trajectory file.
    atom_style : str
        Customised LAMMPS `atom_style` information. Only works with
        `topology_format = data`
    dimensions : iterable of floats
        vector that contains unit cell lengths and probable angles.
        Expected shapes are eithere (6, 0) or (1, 6) or for
        shapes of (3, 0) or (1, 3) all angles are set to 90 degrees.

    Raises
    ------
    IndexError
        If the dimesions of the `dimensions` argument are not 3 or 6.

    Returns
    -------
    `MDAnalysis.Universe`
    """
    universe = mda.Universe(topology,
                            topology_format=topology_format,
                            atom_style=atom_style)

    if coordinates is not None:
        universe.load_new(coordinates, format=trajectory_format)

    if dimensions is not None:
        if len(dimensions) == 3:
            dimensions = [*dimensions, 90, 90, 90]
        elif len(dimensions) != 6:
            raise IndexError(
                "The dimensions must contain at least 3 entries for "
                "the box length and possibly 3 more entries for the angles.")

        universe.trajectory.add_transformations(set_dimensions(dimensions))

    return universe


def run_analsis(analysis_callable,
                mandatory_analysis_parameters,
                optional_analysis_parameters=None,
                reference_universe_parameters=None,
                run_parameters=None,
                output_parameters=None):
    """Perform main client logic.

    Parameters
    ----------
    analysis_callable : function
        Analysis class for which the analysis is performed.
    mandatory_analysis_parameters : dict
        Mandatory parameters for executing the analysis
    optional_analysis_parameters : dict
        Optional parameters for executing the analysis
    run_parameters : dict
        time frame parameters: start, stop, step, verbose
    output_parameters : dict
        output_prefix and output_directory

    Returns
    -------
    `MDAnalysis.analysis.base.AnalysisBase`
        AnalysisBase instance of the given ``analysis_callable`` after run.
    """
    if optional_analysis_parameters is None:
        optional_analysis_parameters = {}

    if run_parameters is None:
        run_parameters = {}

    if output_parameters is None:
        output_parameters = {}

    verbose = run_parameters.pop("verbose", False)

    if reference_universe_parameters is not None:
        reference_universe = create_universe(**reference_universe_parameters)
    else:
        reference_universe = None

    # Initilize analysis callable
    universe = convert_analysis_parameters(analysis_callable,
                                           mandatory_analysis_parameters,
                                           reference_universe)

    convert_analysis_parameters(analysis_callable,
                                optional_analysis_parameters,
                                reference_universe)

    if universe is None:
        universe = reference_universe

    ac = analysis_callable(**mandatory_analysis_parameters,
                           **optional_analysis_parameters)

    # Run the analysis
    for key, value in run_parameters.items():
        run_parameters[key] = convert_str_time(value, universe.trajectory.dt)

    ac.run(verbose=verbose, **run_parameters)

    # Save results
    try:
        ac.save()

    except AttributeError:
        directory = output_parameters.get("output_directory", "")
        fname = output_parameters.get("output_prefix", "")
        fname = f"{fname}_{analysis_callable.__name__}" if fname \
            else analysis_callable.__name__
        save(ac.results, os.path.join(directory, fname))

    return ac


def convert_analysis_parameters(analysis_callable,
                                analysis_parameters,
                                reference_universe=None):
    """
    Convert parameters from the command line suitable for anlysis.

    Special types (i.e AtomGroups, Universes) are converted from the command
    line strings into the correct format. Parameters are changed inplace.
    Note that only keys are converted and no new key are added if
    present in the doc of the `analysis_callable` but not
    in the `analysis_parameters` dict.

    The following types are converted:

    * AtomGroup: Select atoms based on ``universe.select_atoms``
    * Universe: Created from parameters.

    Parameters
    ----------
    analysis_callable : function
        Analysis class for which the analysis should be performed.
    analysis_parameters : dict
        parameters to be processed
    reference_universe : `MDAnalysis.Universe`
        Universe from which the AtomGroup selection are done.

    Returns
    -------
    universe : Universe
        The universe created from the anaylysis parameters or None
        of no ine is created

    Raises
    ------
    ValueError
        If an Atomgroup does not contain any atoms
    """
    params = parse_docs(analysis_callable)[2]
    universe = None

    # If a Universe is part of the parameters several extra arguments with
    # non matching names were created. We seperate them by their connecting
    # character.
    analysis_parameters_keys = [p.split("_")[-1] for p
                                in analysis_parameters.keys()]

    for param_name, dictionary in params.items():
        if param_name in analysis_parameters_keys:
            if "AtomGroup" == dictionary['type']:
                sel = analysis_parameters[param_name]
                atomgrp = reference_universe.select_atoms(sel)
                if atomgrp:
                    analysis_parameters[param_name] = atomgrp
                else:
                    raise ValueError(f"AtomGroup `-{param_name}` with "
                                     f"string of the selection {sel}` "
                                     f"does not contain any atoms.")
            elif "list[AtomGroup]" == dictionary['type']:
                for i, sel in enumerate(analysis_parameters[param_name]):
                    atomgrp = reference_universe.select_atoms(sel)
                    if atomgrp:
                        analysis_parameters[param_name][i] = atomgrp
                    else:
                        raise ValueError(f"AtomGroup `-{param_name}` with "
                                         f"string of the selection {sel}` "
                                         f"does not contain any atoms.")

            elif "Universe" == dictionary['type']:
                # Create universe parameter dictionary from signature
                sig = inspect.signature(create_universe)
                universe_parameters = dict(sig.parameters)

                for k in universe_parameters.keys():
                    universe_parameters[k] = analysis_parameters.pop(
                        f"{k}_{param_name}")

                universe = create_universe(**universe_parameters)
                analysis_parameters[param_name] = universe

    return universe


def setup_clients(ap, title, members):
    """
    Set up analysis clients for an ArgumentParser instance.

    Parameters
    ----------
    ap : `argparse.ArgumentParser`
        Argument parser instance
    title : str
        title of the parser
    members : list
        list containing Analysis classes for setting up the parser
    """
    cli_subparser = ap.add_subparsers(title=title)

    analysis_interfaces = {
        member.__name__: parse_callable_signature(member)
        for member in members
        }

    # adds each Analysis class/function as a CLI under 'cli_subparser'
    # to be writen
    for member_name, parameters in analysis_interfaces.items():
        create_cli(sub_parser=cli_subparser,
                   interface_name=member_name.lower(),
                   parameters=parameters)


def init_base_argparse(name, version, description):
    """Create a basic `ArgumentParser`.

    The parser has options for printing the version, running in debug mode
    and with a logfile. Note that the funtion only adds the options to
    the parser but not the logic for actually running in debug mode nor
    how to store the log file.

    Parameters
    ----------

    name : str
        Name of the cli program

    version : str
        Version of the cli program

    description : str
        Description of the cli program

    Returns
    -------
    `ArgumentParser`
    """
    ap = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    ap.add_argument(
        '--version',
        action='version',
        version=f"{name} {version}",
        )

    ap.add_argument(
        '--debug',
        action='store_true',
        help="Run with debug options.",
        )

    ap.add_argument('--logfile',
                    dest='logfile',
                    action='store',
                    help='Logfile (optional)')
    return ap



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-

# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later
"""Manage data saving."""
import json
import os
import sys
import zipfile
from functools import partial
from pathlib import Path

import numpy as np
from MDAnalysis.analysis.base import Results


def save(results, fprefix="mdacli_results"):
    """
    Save the attributes of a results instance to disk.

    1D, 2D and 3D numpy arrays are saved to csv files. 1D arrays of the
    same length are vertically stacked to create a table. 2D arrays are
    saved directly. 3D arrays are split into 2D arrays along the
    shortest dimension and one CSV is saved for each 2D array created,
    and resulting CSVs are stored together in a ZIP file. Note: higher
    dimensional arrays are ignored.

    We try to save everything else in a JSON file. Non-serializable
    types are ignored.

    Parameters
    ----------
    fprefix : str
        prefix for all files saved

    results : `~MDAnalysis.analysis.base.Results`
        A Results instance from which the stored data is taken.
    """
    save_1D_arrays(results, fprefix=fprefix, remove=True)
    save_2D_arrays(results, fprefix=fprefix, remove=True)
    save_3D_arrays(results, fprefix=fprefix, remove=True)
    save_higher_dim_arrays(results, fprefix=fprefix, remove=True, min_ndim=4)
    save_json_serializables(results, remove=True, fname=fprefix)
    save_Results_object(results, fprefix=fprefix, remove=True)
    return


def save_1D_arrays(results, fprefix="1darray", remove=True):
    """
    Save 1D arrays from results.

    Parameters
    ----------
    results : dict-like
        Dictionary containing results.

    remove : bool
        If true remove keys mapping to 1D numpy arrays.
    """
    list_1D, list_1D_labels = get_1D_arrays(results)

    if not list_1D:
        return

    out_lists, out_lables = stack_1d_arrays_list(list_1D, list_1D_labels)

    for out_list, out_label in zip(out_lists, out_lables):
        out_label = out_label.flatten()

        # [3:] to align lables with entries
        savetxt_w_command(
            fname=f"{fprefix}_{'_'.join(out_label)}.csv",
            X=out_list.T,
            header=''.join([f"{i:>25}" for i in out_label])[3:]
            )

    return return_with_remove(results, list_1D_labels, remove)


def get_1D_arrays(results):
    """Get items from dict which correspond to np.ndarrays one dim."""
    list_1D = []
    list_1D_labels = []

    for key, value in results.items():
        if is_1d_array(value):
            list_1D.append(value)
            list_1D_labels.append(key)

    return list_1D, list_1D_labels


def stack_1d_arrays_list(list_1D, extra_list=None):
    """Stack a list of 1D numpy arrays of the same length vertically together.

    The result is a list containing 2D arrays where each array got the same
    number of rows.

    Parameters
    ----------
    list_1d : list
        list of 1 dimensional numpy arrays

    extra_list : list
        additional list of numpy arrays on which the
        operations are executed as for ``list_1d``

    Returns
    -------
    out_list : list
        list of stacked 2D numpy arrays organized by their length
    out_extra : list
        list of stacked 2D numpy applied applied to the same operations
        as out_list
    """
    # Sort for lengths
    lengths = np.array([len(a) for a in list_1D])
    sorted_idx = np.argsort(lengths)

    # Sort lists according to the lengths of the items
    list_1D_sorted = [list_1D[i] for i in sorted_idx]

    # Count the number of items for each length
    counts = np.unique(lengths, return_counts=True)[1]
    new_length_idx = np.hstack([[0], np.cumsum(counts)])

    out_lists = []
    # Concentanate lists of the same lenngth
    for i in range(0, len(new_length_idx) - 1):
        out_lists.append(np.vstack(list_1D_sorted[new_length_idx[i]:
                                                  new_length_idx[i + 1]]))

    if extra_list is not None:
        extra_list_sorted = [extra_list[i] for i in sorted_idx]
        out_extra = []
        for i in range(0, len(new_length_idx) - 1):
            out_extra.append(np.vstack(extra_list_sorted[new_length_idx[i]:
                                                         new_length_idx[i + 1]]
                                       ))

        return out_lists, out_extra
    else:
        return out_lists


def save_2D_arrays(results, fprefix="2Darr", remove=True):
    """Save items of 2D array."""
    keys = []
    for key, value in results.items():
        value = try_to_squeeze_me(value)
        if is_2d_array(value):
            savetxt_w_command(fname=f"{fprefix}_{key}.csv", X=value)
            keys.append(key)

    return return_with_remove(results, keys, remove)


def save_3D_arrays(results, fprefix="3Darr", remove=True):
    """Save items of 2D array."""
    keys = []
    for key, value in results.items():
        value = try_to_squeeze_me(value)
        if is_3d_array(value):
            save_3D_array_to_2D_csv(
                value,
                arr_name=f"{fprefix}_{key}",
                zipit=True,
                )
            keys.append(key)

    return return_with_remove(results, keys, remove)


def save_3D_array_to_2D_csv(
        item,
        arr_name='arr',
        zipit=True,
        ):
    """
    Save 3D array to 2D CSVs.

    Has option to store all in a ZIP file.
    """
    min_dim = np.argmin(item)
    files_to_zip = []

    # Split array along the dimension with smallest number
    # of entries
    splitted_item = np.split(
        item,
        item.shape[min_dim],
        axis=min_dim,
        )

    save_to_folder = not zipit
    folder = None
    if save_to_folder:
        folder = Path(arr_name)
        folder.mkdir(parents=True, exist_ok=True)

    for i, arr in enumerate(splitted_item):
        fname = f"{arr_name}_dim_{min_dim}_idx_{i}.csv"
        foutname = folder.joinpath(fname) if folder else fname
        savetxt_w_command(fname=foutname, X=np.squeeze(arr))
        files_to_zip.append(fname)

    if zipit:
        save_files_to_zip(files_to_zip, zipname=arr_name, remove=True)

    return


def save_higher_dim_arrays(results, fprefix="XDarr", remove=True, min_ndim=4):
    """Save items of multidimensional arrays to CSV."""
    keys = []
    for key, value in results.items():
        value = try_to_squeeze_me(value)
        if is_higher_dimension_array(value, min_ndim):
            save_result_array(value, fprefix=fprefix, arr_name=key)
            keys.append(key)

    return return_with_remove(results, keys, remove)


def save_result_array(arr, fprefix='prefix'):
    """Save array to disk accoring to num of dimensions."""
    item = np.squeeze(arr)

    save_options = {
        item.ndim == 1: save_1D_arrays,
        item.ndim == 2: save_2D_arrays,
        item.ndim == 3: save_3D_arrays,
        item.ndim > 3: save_higher_dim_arrays,
        }

    save_options[True](item, fprefix=fprefix)
    return


def save_json_serializables(results, remove=True, **jsonargs):
    """Save serializable items to a JSON."""
    json_dict = {
        key: value
        for key, value in results.items()
        if is_serializable(value)
        }

    if remove:
        for key in json_dict.keys():
            results.pop(key)

    if json_dict:
        json_dict["command"] = get_cli_input()
        save_to_json(json_dict, **jsonargs)


def is_serializable(value):
    """Assert if value is json serializable."""
    try:
        json.dumps(value)
        return True
    except (TypeError, OverflowError):
        return False


def save_to_json(json_dict, fname='jdict', indent=4, sort_keys=True):
    """Save dictionary to JSON file."""
    with open(f'{fname}.json', 'w') as f:
        json.dump(json_dict, f, indent=indent, sort_keys=sort_keys)


def save_Results_object(results, fprefix='results', remove=True):
    """Save results if they are Results objects."""
    keys = []
    for key, value in results.items():
        if isinstance(value, Results):
            save(f"{fprefix}_{key}", value)
            keys.append(key)

    return return_with_remove(results, keys, remove)


def return_with_remove(ddict, keys, remove):
    """
    Serve all saving functions.

    If remove is true,
    Returns subset of keys from dict.
    Removes keys subset from original dict.

    Else, return None.
    """
    if remove:
        return {key: ddict.pop(key) for key in keys}

    else:
        return None


def save_files_to_zip(files, zipname='thezip', remove=True):
    """
    Compress all files into a single zip archive.

    Parameters
    ----------
    files : list-like
        File names to save to the ZIP archive.

    zipname : str
        The name of the zip file without extension.

    remove : bool, option, default True
        Removes the original files.
    """
    with zipfile.ZipFile(f'{zipname}.zip', 'w') as zipF:
        for file_name in files:
            zipF.write(
                file_name,
                compress_type=zipfile.ZIP_DEFLATED,
                )

    if remove:
        remove_files(files)

    return


def is_dimension_array(arr, ndim):
    """Assert value is array and of certain dimension."""
    valid = \
        isinstance(arr, np.ndarray) \
        and arr.ndim == ndim

    return valid


def is_higher_dimension_array(arr, ndim):
    """Assert value is array and of certain dimension."""
    valid = \
        isinstance(arr, np.ndarray) \
        and arr.ndim > ndim

    return valid


is_1d_array = partial(is_dimension_array, ndim=1)
is_2d_array = partial(is_dimension_array, ndim=2)
is_3d_array = partial(is_dimension_array, ndim=3)


def try_to_squeeze_me(arr):
    """Squeeze the arr if is array."""
    return np.squeeze(arr) if isinstance(arr, np.ndarray) else arr


def remove_files(files):
    """Remove files from disk."""
    for filename in files:
        os.remove(filename)
    return


def savetxt_w_command(fname, X, header='', fsuffix=".csv", **kwargs):
    """
    Save CSV data with info about execution command.

    Adds the command line input to the header and checks for a doubled
    defined filesuffix.
    """
    header = "{}\n{}".format(get_cli_input(), header)
    fname = "{}{}".format(fname, (not fname.endswith(fsuffix)) * fsuffix)
    np.savetxt(
        fname,
        X,
        header=header,
        fmt="%-20s",
        **kwargs)


def get_cli_input():
    """Return a proper fomatted string of the command line input."""
    program_name = os.path.basename(sys.argv[0])
    # Add additional quotes for connected arguments.
    arguments = [
        '"{}"'.format(arg).strip()
        if " " in arg else arg
        for arg in sys.argv[1:]
        ]

    return "Command line was: {} {}".format(program_name, " ".join(arguments))



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later

"""The toplevel command line interface."""
import logging
import sys
import traceback
import warnings

from MDAnalysis.analysis.base import AnalysisBase

from .colors import Emphasise
from .libcli import (
    find_cls_members,
    init_base_argparse,
    run_analsis,
    setup_clients,
    split_argparse_into_groups,
    )
from .logger import setup_logging
from .utils import _exit_if_a_is_b


logger = logging.getLogger(__name__)


def cli(name,
        module_list,
        base_class=AnalysisBase,
        version="",
        description="",
        skip_modules=None,
        ignore_warnings=False):
    """Create the command-line interface.

    This function creates a command line interface with a given `name` based
    on a module list.

    Parameters
    ----------
    name : str
        name of the interface
    module_list : list
        list of module from which the cli is build up.
    base_class : cls
        Class or list of classes that the Anaylsis modules belong to
    description : str
        description of the cli
    skip_modules : list
        list of strings containing modules that should be ommited
    ignore_warnings : bool
        ignore warnings when importing modules

    Examples
    --------
    This example code creates the command line interface of MDAnalysis::

        from MDAnalysis.analysis import __all__

        import mdacli

        skip_mods = ['AnalysisFromFunction',
                    'HydrogenBondAnalysis',
                    'WaterBridgeAnalysis',
                    'Contacts',
                    'Dihedral',
                    'PersistenceLength',
                    'InterRDF_s']

        mdacli.cli(name="MDAnalysis",
                   module_list=[f'MDAnalysis.analysis.{m}' for m in __all__],
                   version=mdacli.__version__,
                   description=__doc__,
                   skip_modules=skip_mods,
                   ignore_warnings=True)
    """
    modules = find_cls_members(base_class,
                               module_list,
                               ignore_warnings=ignore_warnings)

    skip_modules = [] if skip_modules is None else skip_modules
    modules = [mod for mod in modules if mod.__name__ not in skip_modules]
    _exit_if_a_is_b(modules, None, "No analysis modules founds.")

    ap = init_base_argparse(name=name,
                            version=version,
                            description=description)

    if len(sys.argv) < 2:
        ap.error("A subcommand is required.")

    # There is to much useless code execution done here:
    # 1. We do not have to setup all possible clients all the time.
    #    i.e. for `mda RMSD` only the RMSD client should be build.
    # 2. for something like `mdacli -h` We do not have to build every
    #   sub parser in complete detail.
    setup_clients(ap, title=f"{name} Analysis Modules", members=modules)

    # Be case insensitive for the subcommand
    sys.argv[1] = sys.argv[1].lower()

    args = ap.parse_args()

    if args.debug:
        args.verbose = True
    else:
        # Ignore all warnings if not in debug mode
        warnings.filterwarnings("ignore")

    with setup_logging(logger, logfile=args.logfile, debug=args.debug):
        # Execute the main client interface.
        try:
            analysis_callable = args.analysis_callable

            # Get the correct ArgumentParser instance from all subparsers
            # `[0]` selects the first subparser where our analysises live in.
            _key = analysis_callable.__name__.lower()
            ap_sup = ap._subparsers._group_actions[0].choices[_key]
            arg_grouped_dict = split_argparse_into_groups(ap_sup, args)

            # Some parameters may not exist
            arg_grouped_dict.setdefault("Optional Parameters", {})
            arg_grouped_dict.setdefault("Reference Universe Parameters", None)
            arg_grouped_dict.setdefault("Output Parameters", {})

            run_analsis(analysis_callable,
                        arg_grouped_dict["Mandatory Parameters"],
                        arg_grouped_dict["Optional Parameters"],
                        arg_grouped_dict["Reference Universe Parameters"],
                        arg_grouped_dict["Analysis Run Parameters"],
                        arg_grouped_dict["Output Parameters"])
        except Exception as e:
            if args.debug:
                traceback.print_exc()
            else:
                sys.exit(Emphasise.error(f"Error: {e}"))



# -*- coding: utf-8 -*-
"""Config file for Sphinx-docs."""
from __future__ import unicode_literals

import os
import mock
import sys

import msmb_theme
import sphinx_rtd_theme


# activate if there are dependencies
mock_modules = [
    'matplotlib',
    'MDAnalysis',
    'MDAnalysis.analysis',
    'MDAnalysis.analysis.base',
    'MDAnalysis.transformations',
    'MDAnalysis.transformations.boxdimensions',
    'numpy',
    ]

for modulename in mock_modules:
    sys.modules[modulename] = mock.Mock()

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.coverage',
    'sphinx.ext.doctest',
    'sphinx.ext.extlinks',
    'sphinx.ext.ifconfig',
    'sphinx.ext.intersphinx',
    'sphinx.ext.napoleon',
    'sphinx.ext.todo',
    'sphinx.ext.viewcode',
    'sphinx.ext.autosectionlabel',
    'sphinx_rtd_theme',
    'sphinx_sitemap',
    ]

# for sitemap with https://github.com/jdillard/sphinx-sitemap
site_url = "https://mdacli.mdanalysis.org/"

todo_include_todos = True

exclude_patterns = [
    'i_*',
    ]

if os.getenv('SPELLCHECK'):
    extensions += 'sphinxcontrib.spelling',
    spelling_show_suggestions = True
    spelling_lang = 'en_US'
    # https://sphinxcontrib-spelling.readthedocs.io/en/latest/customize.html
    spelling_word_list_filename = ['../spelling_wordlist.txt']

source_suffix = '.rst'
master_doc = 'index'
project = 'mdacli'
year = '2021'
author = 'Philip Loche, Joao MC Teixeira and Oliver Beckstein'
copyright = '{0}, {1}'.format(year, author)
version = release = '0.1.12'

pygments_style = 'trac'
templates_path = ['_templates']

extlinks = {
    'issue': ('https://github.com/MDAnalysis/mdacli/cissues/%s', '#'),
    'pr': ('https://github.com/MDAnalysis/mdacli/pull/%s', 'PR #'),
    }

# codecov io closes connection if host is accessed too repetitively.
# codecov links are ignored here for the same reason there's a sleep
# in the .travis.yml file
# see https://github.com/codecov/codecov-python/issues/158
linkcheck_ignore = [
    r'https://codecov.io/gh/MDAnalysis/mdacli/*',
    ]

html_theme = "msmb_theme"
html_theme_path = [msmb_theme.get_html_theme_path(),
                   sphinx_rtd_theme.get_html_theme_path()]
html_theme_options = {
    'canonical_url': '',
    'logo_only': True,
    'display_version': True,
    'prev_next_buttons_location': 'bottom',
    'style_external_links': False,
    'style_nav_header_background': 'white',
    # Toc options
    'collapse_navigation': True,
    'sticky_navigation': True,
    'navigation_depth': 4,
    'includehidden': True,
    'titles_only': False,
}
html_use_smartypants = True
html_last_updated_fmt = '%b %d, %Y'
html_split_index = False
html_short_title = '%s-%s' % (project, version)
html_baseurl = site_url
html_logo = "_static/logos/mdacli-logo.png"
html_favicon = "_static/logos/mdanalysis-logo.ico"
html_static_path = ['_static']
html_css_files = ['custom.css']
html_use_opensearch = site_url

html_context = {
    'versions_json_url': os.path.join(site_url, "versions.json")
}

napoleon_use_ivar = True
napoleon_use_rtype = False
napoleon_use_param = False

# Configuration for intersphinx: refer to the Python standard library
# and other packages used by mdacli
intersphinx_mapping = {'https://docs.python.org/': None,
                       'https://docs.mdanalysis.org/stable/': None,
                       }



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under the GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later
"""Setuptools-based setup script for tests of mdacli.

For a basic installation just type the command::

  python setup.py develop

"""

from glob import glob
from os.path import basename, dirname, join, splitext

from setuptools import find_packages, setup


def read(*names, **kwargs):
    """Read description files."""
    path = join(dirname(__file__), *names)
    with open(path, encoding=kwargs.get('encoding', 'utf8')) as fh:
        return fh.read()


long_description = '{}\n{}'.format(
    read('README.rst'),
    read(join('docs', 'CHANGELOG.rst')),
    )

setup(
    name='mdacli',
    version='0.1.12',
    description='A command line client for MDAnalysis Analysis classes.',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    license='GPLv3',
    author='PicoCentauri, joaomcteixeira',
    author_email='philip-loche@gmx.de, joaomcteixeira@gmail.com',
    maintainer='PicoCentauri, joaomcteixeira, MDA devs',
    maintainer_email='philip-loche@gmx.de, joaomcteixeira@gmail.com, mdanalysis@numfocus.org',
    url='https://github.com/MDAnalysis/mdacli',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(i))[0] for i in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows ',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Scientific/Engineering',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Topic :: Scientific/Engineering :: Chemistry',
        'Topic :: Scientific/Engineering :: Physics',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Shells',
        ],
    project_urls={
        'Source': 'https://github.com/MDAnalysis/mdacli',
        'Documentation': 'https://mdacli.readthedocs.io/en/latest/',
        'Changelog': 'https://github.com/MDAnalysis/mdacli/blob/main/docs/CHANGELOG.rst',
        'User Group': 'https://groups.google.com/g/mdnalysis-discussion/',
        'Issue Tracker': 'https://github.com/MDAnalysis/mdacli/issues',
        'Discord': 'https://discord.com/channels/807348386012987462/',
        'Blog': 'https://www.mdanalysis.org/blog/',
        'Twitter': 'https://twitter.com/mdanalysis',
        },
    keywords=[
        'Science',
        'Molecular Dynamics',
        'MDAnalysis',
        ],
    python_requires='>=3.7',
    install_requires=[
        'MDAnalysis>=2.0.0',
        ],
    entry_points={
        'console_scripts': [
            'mda= mdacli.__main__:main',
            ]
        },
    )



#!/usr/bin/env python3
# -*- Mode: python; tab-width: 4; indent-tabs-mode:nil; coding:utf-8 -*-
#
# Copyright (c) 2021 Authors and contributors
#
# Released under GNU Public Licence, v2 or any higher version
# SPDX-License-Identifier: GPL-2.0-or-later

"""
Emphaising strings with colors etc.

Taken from https://gist.github.com/tuvokki/14deb97bef6df9bc6553.
"""


class Emphasise:
    """Class for emphaising strings with colors etc.

    Attributes
    ----------
    bold : str
        bold attribute
    underline : str
        underline attribute
    gray : str
        gray color
    red : str
        red color
    green : str
        green color
    yellow : str
        yellow color
    blue : str
        blue color
    pink : str
        pink color
    turquoise : str
        turquoise color
    """

    _endc = '\033[0m'
    bold = '\033[1m'
    underline = '\033[4m'
    gray = '\033[90m'
    red = '\033[91m'
    green = '\033[92m'
    yellow = '\033[93m'
    blue = '\033[94m'
    pink = '\033[95m'
    turquoise = '\033[96m'

    def _emphasise(self, str, style):
        return f"{style}{str}{self._endc}"

    @staticmethod
    def emphasise(str, style):
        """Decorate a ``str`` with desired style.

        The Style could be a color, bold or underline.

        Parameters
        ----------
        message : str
            message to print
        style : str
            emphasising style. See class attributes for available styles

        Example
        -------
        >>> print(Emphasise.emphasise("My colored message", Emphasise.blue))
        """
        style_str = getattr(Emphasise, style)
        return Emphasise._emphasise(Emphasise, str, style_str)

    @staticmethod
    def warning(message):
        """Return a yellow warning.

        Parameters
        ----------
        message : str
            yellow warning to print

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.warning("Potential Danger!"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.yellow)

    @staticmethod
    def error(message):
        """Return a red error.

        Parameters
        ----------
        message : str
            red error to return

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.error("Potential Danger!"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.red)

    @staticmethod
    def ok(message):
        """Return a green ok.

        Parameters
        ----------
        message : str
            green ok to return

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.ok("Yay!"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.green)

    @staticmethod
    def info(message):
        """Return a blue info.

        Parameters
        ----------
        message : str
            blue info to return

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.info("Blue Yay!"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.blue)

    @staticmethod
    def header(message):
        """Return a pink header.

        Parameters
        ----------
        message : str
            pink header to return

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.header("This is great"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.blue)

    @staticmethod
    def debug(message):
        """Return a turquoise debug message.

        Parameters
        ----------
        message : str
            turquoise debug message to return

        Returns
        -------
        decorated_message : str
            decorated message

        Example
        ------
        >>> print(bcolors.debug("a=1"))
        """
        return Emphasise._emphasise(Emphasise, message, Emphasise.turquoise)
